unit Unit80;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, TeeChartGrid, ExtCtrls, TeeProcs, TeEngine, Chart, Series,
  StdCtrls;

type
  TChartData = array[0..2, 0..9] of double;
  TForm80 = class(TForm)
    Chart1: TChart;
    ChartGrid1: TChartGrid;
    Series1: TAreaSeries;
    Panel1: TPanel;
    ListBox1: TListBox;
    Panel2: TPanel;
    Panel3: TPanel;
    comNullSyle: TComboBox;
    Label1: TLabel;
    chkAddExtraStepUps: TCheckBox;
    chkAddExtraStepDns: TCheckBox;
    Label2: TLabel;
    comData: TComboBox;
    Label3: TLabel;
    comDrawChartMethod: TComboBox;
    procedure chkAddExtraStepUpsClick(Sender: TObject);
    procedure comNullSyleChange(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure rbtnData1Click(Sender: TObject);
  private
    FLoading: boolean;
    function GetChartData: TChartData;
    procedure UpdateChartGrid;
    function GetNullsStyle: TTreatNullsStyle;
    function CreateChartSeries(const aIdx: integer; const aMultiArea: TMultiArea): TAreaSeries;
    procedure UpdateChartMethod1;
    procedure UpdateChartMethod2;
    procedure AfterUpdateChart;
    { Private declarations }
  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;
    procedure UpdateChart;
  end;

var
  Form80: TForm80;

implementation

{$R *.dfm}

uses Math, TypInfo;

{ TForm80 }

const
  CHART_DATA1: TChartData =
    ((  9, 9, 1, 0, 0, 0, 0, 0, 1, 10),
     (  1, 1, 9, 10, 10, 10, 10, 10, 9, 0),
     (  0, 0, 0, 0, 0, 0, 0, 0, 0, 0));{}

  CHART_DATA2: TChartData =
    (( 11, 2, 2, 2, 1, 0, 0, 2, 2, 2),
     (  0, 9, 9, 9, 9, 9, 9, 9, 9, 9),
     (  0, 0, 0, 0, 1, 0, 0, 1, 0, 0));{}

  CHART_DATA3: TChartData =
    ((100, 103, 20, 10, 0, 0, 0, 0, 0, 0),
      (0, 0, 86, 20, 20, 30, 0, 0, 0, 0),
      (0, 0, 0, 79, 93, 86, 119, 123, 127, 130));

  CHART_DATA4: TChartData =
    ((90, 92, 94,  50,   0,   0,   0,   0,   0,   0),
     (10, 11, 12,   7,   0,   0,   0,   0,   0,   0),
     ( 0,  0,  0,  55, 115, 119, 122, 126, 130, 134));



function TForm80.GetChartData: TChartData;
begin
  if comData.ItemIndex = 1 then
    result := CHART_DATA2
  else
  if comData.ItemIndex = 2 then
    result := CHART_DATA3
  else
  if comData.ItemIndex = 3 then
    result := CHART_DATA4
  else
    result := CHART_DATA1
end;

constructor TForm80.Create(aOwner: TComponent);
var
  lTNS: TTreatNullsStyle;
  I: Integer;
begin
  FLoading := True;
  inherited;

  for I := 1 to 4 do
    comData.Items.Add('Data #' + IntToStr(I));
  comData.ItemIndex := 0;

  comDrawChartMethod.Items.Add('Method 1');
  comDrawChartMethod.Items.Add('Method 2');
  comDrawChartMethod.ItemIndex := 0;

  for lTNS := low(TTreatNullsStyle) to High(TTreatNullsStyle) do
    comNullSyle.Items.AddObject(GetEnumName(TypeInfo(TTreatNullsStyle), ord(lTNS)), TObject(lTNS));
  comNullSyle.ItemIndex := 0;
  FLoading := false;

  UpdateChart;
end;

procedure TForm80.chkAddExtraStepUpsClick(Sender: TObject);
begin
  UpdateChart;
end;

procedure TForm80.comNullSyleChange(Sender: TObject);
begin
  if not FLoading then
    UpdateChart;
end;

procedure TForm80.ListBox1Click(Sender: TObject);
begin
  UpdateChartGrid;
end;

procedure TForm80.rbtnData1Click(Sender: TObject);
begin
  UPdateChart;
end;

procedure TForm80.UpdateChartGrid;
begin
  if LIstBox1.ItemIndex > -1 then
    ChartGrid1.Series := TChartSeries(ListBox1.Items.Objects[LIstBox1.ItemIndex]);
end;

function TForm80.GetNullsStyle: TTreatNullsStyle;
begin
  result := TTreatNullsStyle(comNullSyle.Items.Objects[comNullSyle.ItemIndex]);
end;


procedure TForm80.UpdateChart;
begin
  if comDrawChartMethod.ItemIndex = 1 then
    UpdateChartMethod2
  else
    UpdateChartMethod1;
end;

function TForm80.CreateChartSeries(const aIdx: integer; const aMultiArea: TMultiArea): TAreaSeries;
begin
  result := TAreaSeries.Create(self);
  result.ParentChart := Chart1;
  result.AreaLinesPen.Visible := false;
  result.Pen.Visible := false;
  result.Title := 'Series #' + IntToSTr(aIdx);
  result.MultiArea := aMultiArea;
  result.TreatNulls := GetNullsStyle;
end;

procedure TForm80.UpdateChartMethod2;
var
  I: integer;
  J: integer;
  K: Integer;
  lSrs: TChartSeries;
  lLastIdx: Integer;
  lIncome: Double;
  lChartData: TChartData;
begin
  lChartData := GetChartData;
  Chart1.FreeAllSeries;
  for I := High(lChartData) downto Low(lChartData) do
  begin
    lsrs := CreateChartSeries(I, maNone);;

    lLastIdx := High(lChartData[I]);
    for J := 0 to lLastIdx do
    begin

      lIncome := 0;
      for K := 0 to I do
        lIncome := lIncome + lChartData[K, J];  

      //see if previous month income was zero, if so, add 0 here

      if (lIncome > 0) then
        lsrs.AddXY(J, lIncome, IntToStr(J))
      else
        lsrs.AddNullXY(J, 0, IntToStr(J));

    end;
  end;


  AfterUpdateChart;
end;

procedure TForm80.AfterUpdateChart;
var
  I: Integer;
begin
  Caption := '';
  for I := 0 to Chart1.SeriesCount -1 do
    Caption := Caption + '; ' + Chart1[I].Title  + ': ' + IntToStr(Chart1[I].Count);
  Caption := Copy(Caption, 3, MaxInt);



  LIstBox1.Items.Clear;
  for I := 0 to Chart1.SeriesCount -1 do
    LIstBox1.Items.AddObject(Chart1[I].Title, Chart1[I]);
  ListBox1.ItemIndex := 0;
  UpdateChartGrid;
end;


procedure TForm80.UpdateChartMethod1;
var
  I: integer;
  J: integer;
  lLastValue: Extended;
  lSrs: TChartSeries;
  lLastIdx: Integer;
  lIncome: Double;
  lPt: TPoint;
  lStairs: array of TPoint;
  lChartData: TChartData;
begin
  lChartData := GetChartData;
  Chart1.FreeAllSeries;
  for I := Low(lChartData) to High(lChartData) do
  begin
    lsrs := CreateChartSeries(I, maStacked);

    lLastValue := 0.0;

    lLastIdx := High(lChartData[I]);
    for J := 0 to lLastIdx do
    begin
      lIncome := lChartData[I, J];

      //see if previous month income was zero, if so, add 0 here

      if (lIncome > 0) then
      begin
        if chkAddExtraStepUps.Checked and
          (J > 0) and
          (lLastValue = 0) then
        begin
          //no value for previous item, step into current value
          lsrs.AddXY(J, 0, IntToStr(J));

          SetLength(lStairs, Length(lStairs) + 1);
          lStairs[Length(lStairs)-1] := Point(I, J);
        end;{}

        lsrs.AddXY(J, lIncome, IntToStr(J));

        if chkAddExtraStepDns.Checked and
          (J < lLastIdx) and
          (lChartData[I, J+1] = 0) then
        begin
          //no value for next item, step chart down to zero
          lsrs.AddXY(J, 0, IntToStr(J));

          SetLength(lStairs, Length(lStairs)+1);
          lStairs[Length(lStairs)-1] := Point(I, Max(J+1, 0));
        end;{}
      end
      else
        lsrs.AddNullXY(J, 0, IntToStr(J));

      lLastValue := lIncome;
    end;
  end;

  for I := Length(lStairs) - 1 downto 0 do
  begin
    lPt := lStairs[I];
    for J := 0 to Chart1.SeriesCount -1 do
      if J <> lPt.X then
      begin
        lsrs := Chart1[J];
        lsrs.AddXY(lSrs.XValue[lPt.Y], lSrs.YValue[lPt.Y]);
      end;
  end;

  AfterUpdateChart;
end;

end.
